@@ -29,6 +29,8 @@ from pymongo import ASCENDING, DESCENDING, OFF, SLOW_ONLY, ALL
 from pymongo.connection import Connection
 from pymongo.collection import Collection
 from pymongo.dbref import DBRef
+from pymongo.code import Code
+from pymongo.son_manipulator import AutoReference, NamespaceInjector
 from test_connection import get_connection
 
 class TestDatabase(unittest.TestCase):
@@ -242,11 +244,15 @@ class TestDatabase(unittest.TestCase):
         self.assertRaises(TypeError, db.dereference, None)
 
         self.assertEqual(None, db.dereference(DBRef("test", ObjectId())))
-
         obj = {"x": True}
         key = db.test.save(obj)
         self.assertEqual(obj, db.dereference(DBRef("test", key)))
 
+        self.assertEqual(None, db.dereference(DBRef("test", 4)))
+        obj = {"_id": 4}
+        db.test.save(obj)
+        self.assertEqual(obj, db.dereference(DBRef("test", 4)))
+
     def test_eval(self):
         db = self.connection.pymongo_test
         db.test.remove({})
@@ -265,6 +271,11 @@ class TestDatabase(unittest.TestCase):
         self.assertEqual(5, db.eval("function () {return 5;}"))
         self.assertEqual(5, db.eval("2 + 3;"))
 
+        self.assertEqual(5, db.eval(Code("2 + 3;")))
+        self.assertEqual(None, db.eval(Code("return i;")))
+        self.assertEqual(2, db.eval(Code("return i;", {"i": 2})))
+        self.assertEqual(5, db.eval(Code("i + 3;", {"i": 2})))
+
         self.assertRaises(OperationFailure, db.eval, "5 ++ 5;")
 
     # TODO some of these tests belong in the collection level testing.
@@ -351,5 +362,57 @@ class TestDatabase(unittest.TestCase):
             for _ in db.test.find():
                 break
 
+    def test_auto_ref_and_deref(self):
+        db = self.connection.pymongo_test
+        db.add_son_manipulator(AutoReference(db))
+        db.add_son_manipulator(NamespaceInjector())
+
+        db.test.a.remove({})
+        db.test.b.remove({})
+        db.test.c.remove({})
+
+        a = {"hello": u"world"}
+        db.test.a.save(a)
+
+        b = {"test": a}
+        db.test.b.save(b)
+
+        c = {"another test": b}
+        db.test.c.save(c)
+
+        a["hello"] = "mike"
+        db.test.a.save(a)
+
+        self.assertEqual(db.test.a.find_one(), a)
+        self.assertEqual(db.test.b.find_one()["test"], a)
+        self.assertEqual(db.test.c.find_one()["another test"]["test"], a)
+        self.assertEqual(db.test.b.find_one(), b)
+        self.assertEqual(db.test.c.find_one()["another test"], b)
+        self.assertEqual(db.test.c.find_one(), c)
+
+    # some stuff the user marc wanted to be able to do, make sure it works
+    def test_marc(self):
+        db = self.connection.pymongo_test
+        db.add_son_manipulator(AutoReference(db))
+        db.add_son_manipulator(NamespaceInjector())
+
+        db.drop_collection("users")
+        db.drop_collection("messages")
+
+        message_1 = {"title": "foo"}
+        db.messages.save(message_1)
+        message_2 = {"title": "bar"}
+        db.messages.save(message_2)
+
+        user = {"name": "marc",
+                "messages": [message_1, message_2]}
+        db.users.save(user)
+
+        message = db.messages.find_one()
+        db.messages.update(message, {"title": "buzz"})
+
+        self.assertEqual("buzz", db.users.find_one()["messages"][0]["title"])
+        self.assertEqual("bar", db.users.find_one()["messages"][1]["title"])
+
 if __name__ == "__main__":
     unittest.main()
